# -*- coding: utf-8 -*-
from setuphelpers import *

""" 
Installation procedure: https://support.mozilla.org/kb/customizing-firefox-using-policiesjson
https://github.com/thundernest/policy-templates
https://github.com/mozilla/policy-templates/
https://klaus-hartnegg.de/gpo/2018-05-26-Firefox-Policies.html

"""

# Defining variables
app_name = "Mozilla Thunderbird"
policies_file = "policies.json"


def install():
    # Customise the package for your own network
    if "-template" in control.package and not params.get("running_as_luti", False):
        error("Please run the update_package and then customize this package for your network.")


    # Installing the package
    for thund in installed_softwares(app_name):
        # Initializing variables
        thund_dir = thund["install_location"]
        thund_dist_dir = makepath(thund_dir, "distribution")
        policies_path = makepath(thund_dist_dir, policies_file)
        basedir_policies_file = makepath(basedir, policies_file)

        # Writing Thunderbird configuration file
        if not isdir(thund_dist_dir):
            mkdirs(thund_dist_dir)
        print("Writing Thunderbird configuration file: %s" % policies_path)
        """ # Merging actual Thunderbird configuration file with defined one (may fail)
        if isfile(policies_path):
            old_data = json_load(policies_path)
            new_data = json_load(basedir_policies_file)
            data = data_merge(old_data,new_data)
            json_write(policies_path, data, indent=2)
        else:
            data = json_load(basedir_policies_file)
            json_write(policies_path, data, indent=2) """

        # Forcing package Thunderbird configuration
        if isfile(policies_path):
            remove_file(policies_path)
            filecopyto(basedir_policies_file, policies_path)
        else:
            filecopyto(basedir_policies_file, policies_path)

    # Removing policies from registry since it bypass the policies.json file (source: https://support.mozilla.org/bm/questions/1236197)
    if reg_key_exists(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Thunderbird"):
        registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Mozilla\Thunderbird", "", force=True)


def uninstall():
    # Uninstalling the package
    for thund in installed_softwares(app_name):
        # Initializing variables
        thund_dir = thund["install_location"]
        thund_dist_dir = makepath(thund_dir, "distribution")
        policies_path = makepath(thund_dist_dir, policies_file)

        # Removing Thunderbird configuration
        if isfile(policies_path):
            remove_file(policies_path)


def audit():
    # Verifying that the configuration is applied
    for thund in installed_softwares(app_name):
        # Initializing variables
        thund_dir = thund["install_location"]
        thund_dist_dir = makepath(thund_dir, "distribution")
        policies_path = makepath(thund_dist_dir, policies_file)
        if not isfile(policies_path):
            # Re-executiong the package
            print("Configuration is NOT applied for %s" % thund["name"])
            return "WARNING"
        else:
            print("Configuration is correctly applied for %s" % thund["name"])
            return "OK"


def json_load(json_file):
    with open(json_file) as read_file:
        data = json.load(read_file)
    return data


def json_write(json_file, data, sort_keys=True, indent=4):
    import codecs

    with codecs.open(json_file, "w", encoding="utf-8") as write_file:
        json.dump(data, write_file, sort_keys=sort_keys, indent=indent)


def data_merge(a, b):
    """merges b into a and return merged result

    NOTE: tuples and arbitrary objects are not handled as it is totally ambiguous what should happen"""
    key = None
    # ## debug output
    # sys.stderr.write("DEBUG: %s to %s\n" %(b,a))
    try:
        if a is None or isinstance(a, str) or isinstance(a, unicode) or isinstance(a, int) or isinstance(a, long) or isinstance(a, float):
            # border case for first run or if a is a primitive
            a = b
        elif isinstance(a, list):
            # lists can be only appended
            if isinstance(b, list):
                # merge lists
                a.extend(b)
            else:
                # append to list
                a.append(b)
        elif isinstance(a, dict):
            # dicts must be merged
            if isinstance(b, dict):
                for key in b:
                    if key in a:
                        a[key] = data_merge(a[key], b[key])
                    else:
                        a[key] = b[key]
    except TypeError as e:
        raise Exception('TypeError "%s" in key "%s" when merging "%s" into "%s"' % (e, key, b, a))
    return a
